package com.mall.auth.controller;

import com.mall.auth.service.AuthService;
import com.mall.common.domain.UserInfo;
import com.mall.common.domain.request.LoginRequest;
import com.mall.common.domain.vo.LoginVo;
import com.mall.common.domain.vo.UserInfoVo;
import com.mall.common.result.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @Author: lzh
 * @CreateTime: 2024-04-22  17:00
 * @Description: TODO
 * @Version: 1.0
 */
@RestController
@Slf4j
public class AuthController {
    @Autowired
    private AuthService authService;

    /**
     * 账号密码登录接口
     * @param loginVo
     * @return Result
     */
    @PostMapping("/login")
    public Result login(@RequestBody LoginVo loginVo){
        Result result = authService.login(loginVo);
        return result;
    }

    /**
     * 发送验证码接口
     * @param loginRequest
     * @return Result
     */
    @PostMapping("send/code/{phone}")
    public Result sendCode(@RequestBody LoginRequest loginRequest){
        Result result = authService.sendCode(loginRequest);
        return result;
    }

    /**
     * 手机号登录接口
     * @param loginRequest
     * @return Result
     */
    @PostMapping("phoneLogin")
    public Result phoneLogin(@RequestBody LoginRequest loginRequest) {
        Result result = authService.phoneLogin(loginRequest);
        return result;
    }

    /**
     * 手机号注册接口
     * @param userInfoVo
     * @return Result
     */
    @PostMapping("phoneRegister")
    public Result phoneRegister(@RequestBody UserInfoVo userInfoVo) {
        Result result = authService.phoneRegister(userInfoVo);
        return result;
    }

    /**
     * 获取用户信息接口
     * @return Result
     */
    @GetMapping("getUserInfo")
    public Result<UserInfo> userInfo() {
        Result<UserInfo> result = authService.userInfo();
        return result;
    }
}
