package com.mall.auth.service.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import com.mall.auth.feign.UserServiceFeign;
import com.mall.auth.service.AuthService;
import com.mall.common.constant.JwtConstants;
import com.mall.common.constant.TokenConstants;
import com.mall.common.domain.UserInfo;
import com.mall.common.domain.request.LoginRequest;
import com.mall.common.domain.response.JwtResponse;
import com.mall.common.domain.vo.LoginVo;
import com.mall.common.domain.vo.UserInfoVo;
import com.mall.common.redis.RedisCache;
import com.mall.common.result.BizException;
import com.mall.common.result.Result;
import com.mall.common.utils.IdUtils;
import com.mall.common.utils.JwtUtils;
import com.mall.common.utils.StringUtils;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

/**
 * @Author: lzh
 * @CreateTime: 2024-04-22  17:01
 * @Description: TODO
 * @Version: 1.0
 */
@Service
@Log4j2
public class AuthServiceImpl implements AuthService {
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private UserServiceFeign userServiceFeign;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public Result login(LoginVo loginVo) {
        Result<UserInfo> result = userServiceFeign.findLogin(loginVo.getUsername(),loginVo.getPassword());
        UserInfo data = result.getData();
        Assert.notNull(
                data,"用户不存在"
        );
//        String password = SecureUtil.md5(
//                loginVo.getPassword()+ "|" + data.getSalt()
//        );
        if (!loginVo.getPassword().equals(data.getPassword())){
            throw new BizException("密码错误");
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,data.getId());
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY+token,data,TokenConstants.EXPIRATION, TimeUnit.HOURS);
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setEndTim(TokenConstants.EXPIRATION);
        return Result.success(jwtResponse);
    }

    @Override
    public Result sendCode(LoginRequest loginRequest) {
        Result<UserInfo> result = userServiceFeign.findPhone(loginRequest.getPhone());
        UserInfo data = result.getData();
        Assert.notNull(
                data,"请注册"
        );
        String code = RandomUtil.randomNumbers(6);
        redisCache.setCacheObject("send_code"+loginRequest.getPhone(),code);
        //rabbitTemplate发消息
//        rabbitTemplate.convertAndSend(RabbitConstants.SEND_CODE_BY_PHONE,"",message -> {
//            message.getMessageProperties().setMessageId(UUID.randomUUID().toString().replaceAll("-",""));
//            return message;
//        });
        return Result.success("发送成功，验证码为："+code);
    }

    @Override
    public Result phoneLogin(LoginRequest loginRequest) {
        Result<UserInfo> result = userServiceFeign.findPhone(loginRequest.getPhone());
        UserInfo data = result.getData();
        Assert.notNull(
                data,"请注册"
        );
        if (StringUtils.isAllBlank(loginRequest.getCode(),loginRequest.getCode())){
            throw new BizException(408,"手机号验证码为空");
        }
        String code = redisCache.getCacheObject("send_code" + loginRequest.getPhone());

        if (StringUtils.isBlank(code)){
            throw new BizException(408,"验证码已过期");
        }
        if (!loginRequest.getCode().equals(code)){
            throw new BizException(408,"验证码错误");
        }
        HashMap<String, Object> map = new HashMap<>();
        String userKey = IdUtils.genId();
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY+token,data,TokenConstants.EXPIRATION, TimeUnit.HOURS);
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setEndTim(TokenConstants.EXPIRATION);
        return Result.success(jwtResponse);
    }

    @Override
    public Result phoneRegister(UserInfoVo userInfoVo) {
        String salt = RandomUtil.randomNumbers(6);
        String password = SecureUtil.md5(userInfoVo.getPassword()+ "|" + salt);
        UserInfo userInfo = UserInfo.builder()
                .username(userInfoVo.getUsername())
                .password(password)
                .salt(salt)
                .nickname(userInfoVo.getNickname())
                .phone(userInfoVo.getPhone())
                .email(userInfoVo.getEmail())
                .header(userInfoVo.getHeader())
                .gender(userInfoVo.getGender())
                .birthday(userInfoVo.getBirthday())
                .city(userInfoVo.getCity())
                .role(userInfoVo.getRole())
                .sign(userInfoVo.getSign())
                .sourceType(userInfoVo.getSourceType())
                .integration(userInfoVo.getIntegration())
                .status(userInfoVo.getStatus())
                .identityFront(userInfoVo.getIdentityFront())
                .identityReverse(userInfoVo.getIdentityReverse())
                .identityAuthentication(userInfoVo.getIdentityAuthentication())
                .checkState(userInfoVo.getCheckState())
                .createTime(new Date())
                .build();
        userServiceFeign.register(userInfo);
        return Result.success("注册成功");
    }

    @Override
    public Result<UserInfo> userInfo() {
        String token = request.getHeader(TokenConstants.LOGIN_TOKEN_KEY);
        if (StringUtils.isBlank(token)){
            throw new BizException(401,"未登录");
        }
        UserInfo userInfo = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + token);
        return Result.success(userInfo);
    }
}