package com.mall.common.handler;

import com.alibaba.fastjson.JSON;
import com.mall.common.result.BizException;
import com.mall.common.result.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Component;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.stream.Collectors;

/**
 * @author DongZl
 * @description: 全局异常处理
 * @Date 2024-1-10 下午 08:49
 */
@Log4j2
@RestControllerAdvice
@Component
public class GlobalExceptionHandler {


    /**
     * 处理自定义异常
     */
    @ExceptionHandler(BizException.class)
    public Result handleRRException(BizException e){
        return Result.error(e.getCode(),e.getMessage());
    }


    @ExceptionHandler(DuplicateKeyException.class)
    public Result handleDuplicateKeyException(DuplicateKeyException e){
        log.error(e.getMessage(), e);
        return Result.error("数据库中已存在该记录");
    }

    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public Result<String> runtimeException (MethodArgumentNotValidException exception){
        log.error("方法异常{}",exception.getMessage(),exception);
        return Result.error(
                JSON.toJSONString(
                        exception.getBindingResult().getAllErrors()
                                .stream()
                                .map(ObjectError :: getDefaultMessage)
                                .collect(Collectors.toList())
                )
        );
    }

    @ExceptionHandler(value = IllegalArgumentException.class)
    public Result<String> illegalArgumentExceptionHandler(IllegalArgumentException exception){
        log.error("请求异常：[{}]", exception.getMessage(), exception);
        return Result.error(exception.getMessage());
    }

    @ExceptionHandler(NullPointerException.class)
    public Result handleNullPointerException(NullPointerException ex) {
        // 处理空指针异常
        log.error("捕获到空指针异常: " + ex.getMessage());
        return Result.error(ex.getMessage());
    }

}
