package com.mall.server.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mall.common.domain.request.SpikesRequest;
import com.mall.common.domain.vo.ActivitySkuVo;
import com.mall.common.redis.RedisCache;
import com.mall.common.result.Result;
import com.mall.server.domain.SkuEntity;
import com.mall.server.domain.SpikesEntity;
import com.mall.server.enumerate.ActivityEnum;
import com.mall.server.mapper.SpikesMapper;
import com.mall.server.service.SkuService;
import com.mall.server.service.SpikesService;
import com.mall.server.service.SpuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.BoundZSetOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 *秒杀serviceImpl
 */
@Service
public class SpikesServiceImpl extends ServiceImpl<SpikesMapper, SpikesEntity>
        implements SpikesService {

    @Autowired
    private SpikesMapper spikesMapper;

    @Autowired
    private SpuService spuService;

    @Autowired
    private SkuService skuService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private RedisTemplate redisTemplate;

    @Transactional
    @Override
    public Result add(SpikesRequest spikesRequest) {
        //秒杀表信息
        int spikesId = spikesMapper.insert(
                SpikesEntity.builder()
                        .spikesImg(spikesRequest.getSpikesImg())
                        .spikesManyImg(spikesRequest.getSpikesManyImg())
                        .spikesTitle(spikesRequest.getSpikesTitle())
                        .spikesThings(spikesRequest.getSpikesThings())
                        .spikesUnit(spikesRequest.getSpikesUnit())
                        .spikesNum(spikesRequest.getSpikesNum())
                        .createTime(spikesRequest.getCreateTime())
                        .endTime(spikesRequest.getEndTime())
                        .spikesState(spikesRequest.getSpikesState())
                        .build()
        );
        //修改sku商品信息
        skuService.updateBatchById(
                spikesRequest.getSkuList().stream().map(c->
                    SkuEntity.builder()
                            .id(c.getSkuId())
                            .inventoryRestrict(c.getInventoryRestrict())
                            .activityPrice(c.getActivityPrice())
                            .spikesId(spikesId).build()
                    ).collect(Collectors.toList())
        );
        //遍历修改后的sku商品信息作转换
        List<ActivitySkuVo> activitySkuVoList = skuService.list(
                new LambdaQueryWrapper<SkuEntity>()
                        .eq(SkuEntity::getSpikesId, spikesId)
        ).stream().map(c ->
                ActivitySkuVo.builder()
                        .id(c.getId())
                        .spuId(c.getSpuId())
                        .name(c.getName())
                        .categoryId(c.getCategoryId())
                        .brandId(c.getBrandId())
                        .defaultImage(c.getDefaultImage())
                        .title(c.getTitle())
                        .subtitle(c.getSubtitle())
                        .price(c.getPrice())
                        .activityPrice(c.getActivityPrice())
                        .weight(c.getWeight())
                        .inventory(c.getInventory())
                        .inventoryRestrict(c.getInventoryRestrict())
                        .spikesId(c.getSpikesId())
                        .bargainId(c.getBargainId())
                        .groupId(c.getGroupId())
                        .startTime(spikesRequest.getCreateTime())
                        .endTime(spikesRequest.getEndTime()).build()).collect(Collectors.toList());
        activitySkuVoList.forEach(c->{
            //redis秒杀信息存在时间
            long expire = c.getEndTime().getTime() - System.currentTimeMillis()/1000/1000;
            redisCache.setCacheObject("spikes_"+c.getId(),c,expire, TimeUnit.MINUTES);
        });
        return Result.success(true,"添加秒杀成功");
    }
}
